//
//  MyCell.m
//  Regex Visualizer
//
//  Created on 7/25/09.
//  Copyright 2009 Michael Sanders. All rights reserved.
//  All code released under the MIT license.
//
#define SUCCESSCOLOR [NSColor greenColor]
#define ERRORCOLOR [NSColor redColor]

#import "MyCell.h"

@implementation MyCell

- (void)dealloc
{
    [highlightedRows release];
    [super dealloc];
}

// Store only a weak reference to the table view to tell which row is
// being drawn.
- (void)setTableView:(NSTableView *)newTableView
{
    tableView = newTableView;
}

- (void)setSuccessRow:(unsigned)row
{
    successRow = row;
}

- (void)highlightRow:(unsigned)row
{
    if (highlightedRows == nil) {
        highlightedRows = [[NSMutableArray alloc] init];
    }

    [highlightedRows addObject:[NSNumber numberWithInt:row]];
    NSAssert(row < [tableView numberOfRows], @"Row out of range");
}

- (void)disableHighlighting
{
    [highlightedRows removeAllObjects];
}

- (void)drawInteriorWithFrame:(NSRect)frame
                       inView:(NSView *)controlView
{
    [super drawInteriorWithFrame:frame inView:controlView];

    if (![highlightedRows count]) return;

    frame = NSInsetRect(frame, 1, 1);
    unsigned currentRow = [tableView rowsInRect:frame].location;
    for (NSNumber *row in highlightedRows) {
        if (currentRow == [row intValue]) {
            if (currentRow == successRow) {
                [SUCCESSCOLOR set];
            } else {
                [ERRORCOLOR set];
            }

            NSBezierPath *highlightPath =
                [NSBezierPath bezierPathWithRoundedRect:frame
                                                xRadius:4.0
                                                yRadius:4.0];
            [highlightPath setLineWidth:2.0];
            [highlightPath stroke];
            // DLog(@"Highlighting cell %@", self);
            break;
        }
    }
}

@end
