//
//  MyTableView.m
//  Regex Visualizer
//
//  Created on 7/25/09.
//  Copyright 2009 Michael Sanders. All rights reserved.
//  All code released under the MIT license.
//

#import "MyTableView.h"
#import "MyCell.h"

@implementation MyTableView

- (void)highlightCellAtColumn:(NSTableColumn *)column
                          row:(int)rowIndex
                   successRow:(unsigned)successRow
              disableLastCell:(BOOL)disableLastCell
{
    NSCell *dataCell = [column dataCell];
    if ([dataCell isKindOfClass:[MyCell class]]) {
        if (disableLastCell) {
            [lastHighlightedCell disableHighlighting];
        }
        lastHighlightedCell = (MyCell *)dataCell;
        [(MyCell *)dataCell setSuccessRow:successRow];
        [(MyCell *)dataCell highlightRow:rowIndex];
        [self setNeedsDisplay:YES];
    }
}

- (void)highlightCellsAtColumnIndex:(int)colIndex
                               rows:(NSArray *)rows
                         successRow:(unsigned)successRow
{
    [lastHighlightedCell disableHighlighting];

    NSTableColumn *column = [[super tableColumns] objectAtIndex:colIndex];
    for (NSNumber *row in rows) {
        [self highlightCellAtColumn:column
                                row:[row intValue]
                         successRow:successRow
                    disableLastCell:NO];
    }
}

- (void)highlightCellsAtColumnIdentifier:(id)identifier
                                    rows:(NSArray *)rows
                              successRow:(unsigned)successRow
{
    int colIndex = [self columnWithIdentifier:identifier];
    if (colIndex == -1) colIndex = 0;
    [self highlightCellsAtColumnIndex:colIndex
                                 rows:rows
                           successRow:successRow];
}

@end
