//
//  TransitionTableData.m
//  Regex Visualizer
//
//  Created on 7/24/09.
//  Copyright 2009 Michael Sanders. All rights reserved.
//  All code released under the MIT license.
//

#import "TransitionTableData.h"
#import "MyCell.h"

@implementation TransitionTableData

- (void)awakeFromNib
{
    MyCell *dataCell = [[MyCell alloc] initTextCell:@""];
    [dataCell setTableView:tableView];
    [[[tableView tableColumns] objectAtIndex:0] setDataCell:dataCell];
    [dataCell release];
}

- (void)dealloc
{
    [transitions release];
    [super dealloc];
}

- (void)setTransitions:(NSArray *)newTransitions
{
    // Retrieve a non-mutable copy of the array
    NSArray *copy = [newTransitions copy];
    [transitions release];
    transitions = copy;

    // Remove all columns past delta when new data is given
    // The array must be copied or else removeTableColumn: will modify it
    NSArray *tableColumns = [[tableView tableColumns] copy];
    unsigned i, columnCount = [tableColumns count];
    for (i = 1; i < columnCount; ++i) {
        [tableView removeTableColumn:[tableColumns objectAtIndex:i]];
    }

    // Add new table columns
    // This is probably inefficient but oh well.
    NSMutableArray *identifiers = [[NSMutableArray alloc] init];
    for (NSDictionary *symbols in transitions) {
        for (NSString *identifier in [symbols allKeys]) {
            if (![identifiers containsObject:identifier]) {
                [identifiers addObject:identifier];
            }
        }
    }

    // Set each column width to be the same as the first column
    float deltaWidth = [[tableColumns objectAtIndex:0] width];

    // Sort columns alpha-numerically
    // [identifiers sortUsingSelector:@selector(compare:)];
    columnCount = [identifiers count];
    for (i = 0; i < columnCount; ++i) {
        NSString *identifier = [identifiers objectAtIndex:i];
        NSTableColumn *tableColumn = [[NSTableColumn alloc] initWithIdentifier:
                                      identifier];
        [tableColumn setWidth:deltaWidth];

        // Center the text in each header column
        NSCell *headerCell = [tableColumn headerCell];
        [headerCell setAlignment:NSCenterTextAlignment];
        [headerCell setStringValue:identifier];

        MyCell *dataCell = [[MyCell alloc] initTextCell:@""];
        [dataCell setTableView:tableView];
        [tableColumn setDataCell:dataCell];
        [dataCell release];

        [tableView addTableColumn:tableColumn];
        [tableColumn release];
    }
    [identifiers release];
    [tableColumns release];
}

#pragma mark TableView dataSource methods

- (id)          tableView:(NSTableView *)tableView
objectValueForTableColumn:(NSTableColumn *)tableColumn
                      row:(int)rowIndex;
{

    if ([[tableColumn identifier] isEqualToString:@"delta"]) {
        return [NSString stringWithFormat:@"%i", rowIndex];
    } else {
        NSArray *states = [[transitions objectAtIndex:rowIndex]
                                         objectForKey:[tableColumn identifier]];
        // Return an empty set if no object was found
        return states == nil ? @"∅" : [states componentsJoinedByString:@", "];
    }
    return nil;
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [transitions count];
}

@end
